/*  Copyright 2009 Marc Toussaint
    email: mtoussai@cs.tu-berlin.de

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a COPYING file of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/> */

/*! \file util.h
    \brief many utilities: contains a bunch of standard includes, macros, operators,
    etc. that are used by most of my code */
#ifndef MT_util_h
#define MT_util_h

//----- I need a system flag!
#if !defined MT_MSVC && !defined MT_Cygwin && !defined MT_Linux && !defined MT_MinGW
#  define MT_Linux
  //#  error please define a system flag (one of MT_MSVC, MT_Linux, MT_Cygwin, or MT_MinGW)! (for MSVC also define /Zm500)
#endif

//----- standard includes:
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <typeinfo>
#include <math.h>
#include <string.h>
//#include <typeinfo>
#if defined MT_MSVC // || defined WIN32 || defined _WIN32 || defined _MSC_VER
//#  include<limits>
#  include<time.h>
#  include<sys/timeb.h> 
#  include<windows.h>
#  undef min
#  undef max
#  define MT_TIMEB
#  define __FUNCTION__ ""
#  ifdef MT_QT
#    undef  NOUNICODE
#    define NOUNICODE
#  endif
#  pragma warning(disable: 4305 4244 4250 4355 4786 4996)
#endif
#if defined MT_MinGW
#  include<unistd.h>
#  include<sys/time.h>
#  include<sys/timeb.h>
#  define MT_TIMEB
#endif
#if defined MT_Linux || defined MT_Cygwin
//#  include<unistd.h>
#  include<sys/time.h>
#  include<sys/times.h>
#  include<sys/resource.h>
#endif

#if defined MT_QTGLUT || defined MT_FREEGLUT
#  ifdef MT_MSVC
#    include<windows.h>
#    undef min //I hate it that windows defines these macros!
#    undef max
#  endif
#  define MT_GL
#  ifdef MT_QTGLUT
#    undef MT_FREEGLUT
#  endif
#endif
#ifdef MT_doxy
#  define DOX !\brief
#endif



//----- declare my namespace for the first time:
//! Marc Toussaint namespace
namespace MT{
  inline void breakPoint(){
    int i;
    i=5;
  }
}


//----- error handling:
#ifdef MT_MSVC
#  define MT_HERE "@"<<(strrchr(__FILE__,'\\')?strrchr(__FILE__,'\\')+1:__FILE__)<<':'<<__LINE__<<':'<<__FUNCTION__<<": "
#else
#  define MT_HERE "@"<<(strrchr(__FILE__,'/')?strrchr(__FILE__,'/')+1:__FILE__)<<':'<<__LINE__<<':'<<__FUNCTION__<<": "
#endif
#ifndef MT_MSG
#  define MT_MSG(msg) { std::cerr <<MT_HERE <<msg <<std::endl; MT::breakPoint(); }
#endif
#ifndef HALT
#  define HALT(msg) { std::cerr <<MT_HERE <<msg <<" --- HALT" <<std::endl; MT::breakPoint(); throw "HALT"; }
#  define NIY HALT("not implemented yet")
#endif


//----- check macros:
#ifndef MT_NOCHECK
#  define CHECK(cond,msg) if(!(cond)) HALT("CHECK failed: "<<msg);
#else
#  define CHECK(cond,msg)
#endif


//----- basic defs:
#define MT_PI 3.14159265358979323846
//#define MT_2PI (2.*3.14159265358979323846)
#define MT_LN2 0.69314718055994528622676398299518041312694549560546875
#define MT_2PI 6.283195307179587
#define MT_LnSqrt2Pi -0.9189385332046727417803296 
#define MT_SQRT2 1.414213562373095049
#define MT_SQRTPI 1.772453850905516027
typedef unsigned char byte;      //!< byte
typedef unsigned short int uint16; //!< 2 bytes
typedef unsigned int uint;       //!< unsigned integer
typedef unsigned long long int uint64; //! ...
typedef enum _FwdBwd { FORWARD, BACKWARD } FwdBwd;
typedef const char *ccp;

//----- macros for looping
#ifndef forall
#define forall(i,a)            for(a.First(i)              ;a.CheckNext(i);    a.Next(i))
#define forall_rev(i,a)        for(a.Last(i)               ;a.CheckPrev(i);    a.Prev(i))
#define forall_save(i,j,a)     for(a.First(i),j=i,a.Next(j);a.CheckNext(i);i=j,a.Next(j))
#define forall_rev_save(i,j,a) for(a.Last(i) ,j=i,a.Prev(j);a.CheckPrev(i);i=j,a.Prev(j))
#endif


//----- macros to define the standard << and >> operatos for most my classes:
#define stdInPipe(type)\
  inline std::istream& operator>>(std::istream& is,type& x){ x.read(is);return is; }
#define stdEqPipe(type)\
  inline type& operator<<(type& x,const char *str){ std::istringstream ss(str); ss >>x; return x; }
#define stdOutPipe(type)\
  inline std::ostream& operator<<(std::ostream& os,const type& x){ x.write(os); return os; }
#define stdPipes(type)\
  inline std::istream& operator>>(std::istream& is,type& x){ x.read(is);return is; }\
  inline std::ostream& operator<<(std::ostream& os,const type& x){ x.write(os); return os; }\
//! assignes the object x to the description given by str
#define fwdStdPipes(type)\
  inline std::istream& operator>>(std::istream& is,type& x);\
  inline std::ostream& operator<<(std::ostream& os,const type& x);


//----- macros for piping doubles EXACTLY (without rounding errors) in hex coding:
#define OUTHEX(y) "0x" <<std::hex <<*((unsigned long*)&y) <<std::dec
#define INHEX(y) std::hex >>*((unsigned long*)&y) >>std::dec


//----- standard conversion base type to a class:
#ifndef MT_doxy
//! the basetype template `casts' an elemental type into a class
template<class T>
class basetype{
public:
  T x; //!< the single datum
  basetype():x(){};
  basetype(const T _x):x(_x){};

  T& data(){ return x; }
  operator T() const{ return x; } //!< implicit and const access
  void read(std::istream& is){ is >>x; } //!< implements standard operator>>
  void write(std::ostream& os) const{ os <<x; } //!< implements standard operator<<
};
template<class T> stdInPipe(basetype<T>);
template<class T> stdOutPipe(basetype<T>);

namespace MT{
  typedef basetype<byte> Byte;
  typedef basetype<int> Int;
  typedef basetype<float> Float;
  typedef basetype<double> Double;
}
#endif



//===========================================================================
//
// a very basic memory buffer
//

namespace MT{
//! minimal memory buffer using memove internally; a simplification of the Mem class
template<class T>
class Mem{
public:
  T *p;    //!< pointer to memory
  uint N;  //!< \# elements
  uint M;  //!< actual buffer size (in terms of # elements)
  uint S;  //!< element size (sizeof(T))
  Mem(){ p=0; S=sizeof(T); N=0; M=0; }
  explicit Mem(uint n){ p=0; S=sizeof(T); N=0; M=0; resize(n); }
  ~Mem(){ if(M) delete[] p; }
  void append(const T& x){ resizeCopy(N+1); operator()(N-1)=x; }
  void remove(uint i,uint n=1){
    if(N>i+n) memmove(p+i,p+i+n,S*(N-i-n));
    resizeCopy(N-n);
  }
  void free(){ if(M) delete[] p; p=0; M=N=0; }
  void zero(){ memset(p,0,S*M); }
  T &operator()(uint i) const{ CHECK(i<N,"Mem range error ("<<i<<"<"<<N<<")"); return p[i]; }
  void resizeCopy(uint n){ resize(n,true); }
  void resize(uint n,bool copy){
    if(N==n) return;
    T *pold=p;
#if 0 //rigid allocation: M always equals exactly N
    if(M!=n){
      if(n) p=new T [n];
      if(!p) HALT("MT::Mem failed memory allocation of "<<n*S<<"bytes");
      if(copy) memmove(p,pold,S*(N<n?N:n));
      if(M) delete[] pold;
      N=M=n;
    }
#else //flexible allocation (more than needed in case of multiple resizes)
    if(n>0 && M==0){ //first time
      p=new T [n];
      if(!p) HALT("MT::Mem failed memory allocation of "<<n*S<<"bytes");
      M=n;
    }else if(n>M || 10+2*n<M/2){
      uint oversize=10+2*n;
      p=new T [oversize];
      if(!p) HALT("MT::Mem failed memory allocation of "<<n*S<<"bytes");
      if(copy) memmove(p,pold,S*(N<n?N:n));
      if(M) delete[] pold;
      M=oversize;
    }
    N=n;
#endif
  }
  Mem& operator=(const Mem& m){
    CHECK(S==m.S,"");
    resize(m.N,false);
    memmove(p,m.p,S*N);
    return *this;
  }
};
}

typedef MT::Mem<double> doubleM;
typedef MT::Mem<float>  floatM;
typedef MT::Mem<uint>   uintM;
typedef MT::Mem<int>    intM;
typedef MT::Mem<char>   charM;
typedef MT::Mem<byte>   byteM;
typedef MT::Mem<bool>   boolM;

//===========================================================================
//
// a shared memory
//

namespace MT{

#define maxShmGuests 20
#define shmMaxBlocks 20
struct SHMinfoBlock{
  const char pagename[20];
  uint size,used;
  uint guests;
  int guestPids[maxShmGuests];
  char blockNames[shmMaxBlocks][20];
  uint blockOffsets[shmMaxBlocks];
  pthread_rwlock_t blockLocks[shmMaxBlocks];
};

struct Lock{
  //documentation: http://cs.pub.ro/~apc/2003/resources/pthreads/uguide/document.htm
  pthread_rwlock_t lock;
  void init(bool process_shared=true){
    pthread_rwlockattr_t   att;
    int r;
    r=pthread_rwlockattr_init(&att); if(r) HALT("");
    if(process_shared){
      r=pthread_rwlockattr_setpshared(&att, PTHREAD_PROCESS_SHARED); if(r) HALT("");
    }
    r=pthread_rwlock_init(&lock, &att); if(r) HALT("");
  }
  void read(){
    int r=pthread_rwlock_rdlock(&lock);
    if(r) HALT("");
  }
  
  void write(){
    int r=pthread_rwlock_wrlock(&lock);
    if(r) HALT("");
  }
  
  void unlock(){
    int r=pthread_rwlock_unlock(&lock);
    if(r) HALT("");
  }
};

struct SHM{
  int systemId;
  uint guestId;
  byte *p;
  bool created,opened;
  SHMinfoBlock *info;
  SHM(){ created=opened=false; p=NULL; systemId=NULL; info=NULL; guestId=-1; }
  void open(const char *pagename,uint size,bool directDestroy=false);
  void close();
  void destroy();
  void readLock(uint i);
  void writeLock(uint i);
  void unlock(uint i);
  void report();
  template<class T> T* newBlock(const char* objName,uint *i=NULL);
};

template<class T> struct Shared{
  uint blockId;
  T *p;
  SHM *shm;
  Shared(SHM& _shm,const char* name){ shm=&_shm; p=shm->newBlock<T>(name,&blockId); }
  void readLock() { shm->readLock(blockId); }
  void writeLock(){ shm->writeLock(blockId); }
  void unlock()   { shm->unlock(blockId); }
  void set(const T& x){ writeLock(); *p=x; unlock(); }
  void get(T& x)      { readLock();  x=*p; unlock(); }
  T& operator()(){ return *p; }
};
}

//===========================================================================
//
// standard little methods in my namespace
//

namespace MT{
  extern int argc;
  extern char** argv;
  extern std::ifstream cfgFile;
  extern bool cfgOpenFlag,cfgLock;
  extern Mem<void*> parameters;
  //! stream modifier for some classes (Mem in particular)
  extern bool IOraw;
  //! no logfile: default=true, becomes false when MT::init is called
  extern bool noLog;

  //----- files
  std::ofstream& log(const char *name="MT.log");
  void open(std::ofstream& fs,const char *name,const char *errmsg="");
  void open(std::ifstream& fs,const char *name,const char *errmsg="");
  template<class T> void save(const T& x,const char *filename);
  template<class T> void load(T& x,const char *filename,bool change_directory=false);
  void decomposeFilename(char *&_path,char *&name,const char *filename);
  
  //----- strings and streams
  bool contains(const char *s,char c);
  char skip(std::istream& is,const char *skipchars=" \n\r\t",bool skipCommentLines=true);
  void skipLine(std::istream& is);
  char peerNextChar(std::istream& is);
  void parse(std::istream& is,const char *str);
  bool skipUntil(std::istream& is,const char *tag);

  //----- functions
  byte bit(byte *str,uint i);
  void flip(byte& b,uint i);
  void flip(int& b,uint i);
  double modMetric(double x,double y,double mod);
  double sign(double x);
  double linsig(double x);
  void   constrain(double& x,double a,double b);
  double phi(double dx,double dy);
  double dphi(double x,double y,double dx,double dy);
  double DIV(double x,double y,bool force=false);
  double sigmoid11(double x);
  double sigmoid(double x);
  double dsigmoid(double x);
  double approxExp(double x);
  double Log(double x);
  uint   Log2(uint n);
  double sqr(double x);
  double sinc(double x);
  double cosc(double x);
  template<class T> T MIN(T a,T b){ return a<b?a:b; }
  template<class T> T MAX(T a,T b){ return a>b?a:b; }

  //----- time access
  double realTime();
  double cpuTime();
  double sysTime();
  double totalTime();
  char *date();
  void wait(double sec);
  bool wait();

  //----- memory
  long mem();

  //----- timer functions
  void timerStart();
  double timerRead(bool reset=false);
  double timerRead(bool reset,double startTime);
  double timerPause();
  void timerResume();

  //----- command line handling
  void initCmdLine(int _argc, char *_argv[]);
  bool checkCmdLineTag(const char *tag);
  char *getCmdLineArgument(const char *tag);

  //----- config file handling
  void openConfigFile(const char *name=0);

  //----- parameter grabbing from command line, config file, or default value
  template<class T> T getParameter(const char *tag);
  template<class T> T getParameter(const char *tag,const T& Default);
  template<class T> void getParameter(T& x,const char *tag,const T& Default);
  template<class T> void getParameter(T& x,const char *tag);
  //template<class T> bool getFromCmdLine(T& x,const char *tag);
  //template<class T> bool getFromCfgFile(T& x,const char *tag);
  //template<class T> void getParameter(T& x,const char *tag);
  //template<class T> void getParameterBase(T& x,const char *tag,bool hasDefault,const T* Default);

#ifndef MT_doxy
#ifdef MT_QT
  struct InitQt{ InitQt(); static char *className(){ return 0; } static void staticMetaObject(){} };
#else
  struct InitQt{};
#endif
#endif
}

std::istream& operator>>(std::istream& is,const char *str);
std::istream& operator>>(std::istream& is,char *str);


//===========================================================================
//
// Parameter class - I use it frequently to read parameters from file or cmd line
// 

#if 1
namespace MT{
  /*!\brief A parameter that initializes itself from the command line
    (use \c MT::init), parameter file, or a default value (priority in
    this order).  Initialization is done on the fly the _first_ time
    its value is queried (i.e., referenced by the cast operators).*/
  template<class type>
  class Parameter{
  public:
    const char *typeName;
    type value,Default;
    const char *tag;
    bool initialized,hasDefault;

  public:
    //!@name constructors

    //! Determines the tag to search for in parameter file/command line
    explicit Parameter(const char *_tag){
      typeName=typeid(type).name();
      MT::parameters.append(this);
      initialized=false;
      tag=_tag;
      hasDefault=false;
    };

    /*!\brief specifies also a default value -- parameter does not have to but
      can be specified in the parameter file/command line */
    Parameter(const char *_tag,const type& _default){
      typeName=typeid(type).name();
      MT::parameters.append(this);
      initialized=false;
      tag=_tag;
      hasDefault=true;
      Default=_default;
    };

    ~Parameter(){
      for(uint i=0;i<MT::parameters.N;i++){
	if(MT::parameters(i)==this){ MT::parameters.remove(i); return; }
      }
    }

    //!@name value access

    //! standard type conversion: returns a const of the parameter value
    operator type(){ if(!initialized) initialize(); return value; }

    //! ()-operator: returns an lvalue of the parameter value
    type& operator()(){ if(!initialized) initialize(); return value; }


    //!@name manipulation

    //! assigs a value to the parameter -- no further initialization needed
    type& operator=(const type v){ initialized=true; value=v; return value; }

    //! set the tag (replacing the one from the constructor)
    void setTag(char *_tag){ tag=_tag; }

    /*!\brief enforces that the parameter is reinitialized from the parameter
      file/command line, the next time it is referenced -- even if it
      has been initialized before */
    void reInitialize(){ initialized=false; }


    //!@name explicit grabbing


  private:
    void initialize();
  };

}
#endif


//===========================================================================
//
// random number generator
//

namespace MT{
  /*!\brief A random number generator. An global instantiation \c
    MT::rnd of a \c Rnd object is created. Use this one object to get
    random numbers.*/
  class Rnd{
  private:
    bool ready;
    int rpoint;     /* Feldindex    */
    long rfield[256];   /* Schieberegisterfeld  */


  public:
    //! ...
    Rnd(){ ready=false; };


  public://!@name initialization
    //! initialize with a specific seed
    unsigned long seed(unsigned long n);

    //! use Parameter<uint>("seed") as seed
    unsigned long seed();

    //! uses the internal clock to generate a seed
    unsigned long clockSeed();

  public://!@name access
    //! a initeger random number uniformly distributed in [0,?]
    uint num(){ if(!ready) seed(); return (uint)rnd250() >> 5; }
    //! same as \c num()
    uint operator ()(){ return num(); }
    //! a initeger random number uniformly distributed in [0,\c i-1]
    uint num(uint limit){
      CHECK(limit,"zero limit in rnd.num()"); return num() % limit; }
    uint num(int lo,int hi){ return lo+num(hi-lo+1); }
    //! same as \c num(i)
    uint operator ()(uint i){ return num(i); }
    uint operator ()(int lo,int hi){ return num(lo,hi); }
    //! a random variable uniformly distributed in [0,1]
    double uni(){ return ((double)num(1<<22))/(1<<22); }
    //! a random variable uniformly distributed in [\c low, \c high]
    double uni(double low,double high){ return low+uni()*(high-low); }
    //! a gaussian random variable with mean zero
    double gauss();
    /*!\brief a positive integer drawn from a poisson distribution with given
      \c mean; is case \c mean>100, a (positive) gauss number
      \c floor(mean+gauss(sqrt(mean))+.5) is returned */
    uint poisson(double mean);
    // @}


  private:
    long  rnd250(){
      rpoint = (rpoint+1) & 255;          // Index erhoehen
      return rfield[rpoint] =  rfield[(rpoint-250) & 255]
	                     ^ rfield[(rpoint-103) & 255];
    }

    void seed250(long seed);
  };

  //! The global Rnd object
  extern Rnd rnd;
}



//===========================================================================
//
// String class
//

namespace MT{
  typedef std::iostream IOStream;
  
  /*!\brief String implements the functionalities of an ostream and an
      istream, but also can be send to an ostream or read from an
      istream. It is based on a simple streambuf derived from the
      MT::Mem class */
  class String:public IOStream{
  private:
    class StringBuf:public std::streambuf{
    public:
      Mem<char> *bufmem;
      String *string;
      virtual int overflow(int C = traits_type::eof()){
        (*bufmem)(bufmem->N-1)=C; bufmem->append(0); resetI();
        return C;
      }
      virtual int sync(){
        if(string->flushHandler) string->flushHandler(*string);
        return 0;
      }
      void resetI(){ setIpos(bufmem->p); }
      void setIpos(char *p){ setg(bufmem->p,p,bufmem->p+bufmem->N-1); }
      char *getIpos(){ return gptr(); }
    } buffer;
    
  public:
    Mem<char> mem;
    static const char *readSkipSymbols; //<! default argument to read method (also called by operator>>)
    static const char *readStopSymbols; //<! default argument to read method (also called by operator>>)
    static int readEatStopSymbol;       //<! default argument to read method (also called by operator>>)
    void (*flushHandler)(String&);

    void init(){ buffer.bufmem=&mem; buffer.string=this; flushHandler=NULL; }
    //! standard constructor
    String():IOStream(&buffer){ init(); clr(); }
    //! copy constructor
    String(const String& s):IOStream(&buffer){ init(); this->operator=(s); }
    //! copy constructor for an ordinary C-string (needs to be 0-terminated)
    explicit String(const char *s):IOStream(&buffer){ init(); this->operator=(s); }

    //! returns a reference to this
    IOStream& stream(){ return (IOStream&)(*this); }
    //! returns a reference to this
    String& operator()(){ return *this; }
    /*!\brief returns the true memory buffer (C-string) of this class (which is
      always kept 0-terminated) */
    operator char*(){ return mem.p; }
    //! as above but const
    operator const char*() const{ return mem.p; }
    //! also returns the char-pointer
    const char *p() const{ return mem.p; }
    //! returns the i-th char
    char &operator()(uint i){ return mem(i); }

    //! copy operator
    void operator=(const String& s){ mem=s.mem; buffer.resetI(); }
    //! copies from the C-string
    void operator=(const char *s){ mem.resize(strlen(s)+1,false); memmove(mem.p,s,strlen(s)+1); buffer.resetI(); }
    void set(const char *s,uint n){ mem.resize(n+1,false); memmove(mem.p,s,n); mem(n)=0; buffer.resetI(); }
    //! shorthand for the !strcmp command
    bool operator==(const char *s){ return !strcmp(mem.p,s); }
    //! shorthand for the !strcmp command
    bool operator==(const String& s){ return !strcmp(mem.p,s.mem.p); }

    //! deletes all memory and resets all stream flags
    String& clr(){ resetI(); mem.resize(1,false); mem(0)=0; clear(); return *this; }
    /*!\brief when using this String as an istream (to read other variables
      from it), this method resets the reading-pointer to the beginning
      of the string and also clears all flags of the stream */
    String& resetI(){ buffer.resetI(); clear(); return *this; }
    //! set the reading-pointer to an arbitrary place
    void setIpos(char *p){ buffer.setIpos(p); }
    //! get the reading-pointer
    char *getIpos(){ return buffer.getIpos(); }

    //! returns the length of the string (excluding the terminating 0)
    uint N() const{ return mem.N-1; }
    /*!\brief appends some other text/variable/whatever to the string (as if
      piped into it) */
    template<class T>
    String operator+(const T& v) const{ String news(*this); news <<v; return news; }
    /*!\brief add a prefix to the string */
    template<class T>
    String prepend(const T& v) const{ String news; news <<v <<*this; return news; }

    //! writes the string into some ostream
    void write(std::ostream& os) const{ os <<mem.p; }
    /*!\brief reads the string from some istream: first skip until one of the stopSymbols
      is encountered (default: newline symbols) */
    void read(std::istream& is,const char* skipSymbols=NULL,const char *stopSymbols=NULL,int eatStopSymbol=-1){
      if(!skipSymbols) skipSymbols=readSkipSymbols;
      if(!stopSymbols) stopSymbols=readStopSymbols;
      if(eatStopSymbol==-1) eatStopSymbol=readEatStopSymbol;
      MT::skip(is,skipSymbols);
      mem.resize(10,false);
      uint n=0;
      char c=is.get();
      while(c!=-1 && is.good() && !MT::contains(stopSymbols,c)){
        if(mem.N<=n) mem.resizeCopy(n+10);
        mem(n)=c;
        n++;
        c=is.get();
      }
      if(c==-1) is.clear();
      if(c!=-1 && !eatStopSymbol) is.putback(c);
      //is.clear();
      mem.resizeCopy(n+1);
      mem(n)=0;
      resetI();
    }
  };
}
stdPipes(MT::String);

#define STRING(x) ((MT::String&)(MT::String().stream()<<x)).p()



//===========================================================================
//
// Color class
//

#ifndef MT_doxy
namespace MT{
  //! simple float[3] color class
  class Color{
  public:
    float
      r, //!< red
      g, //!< green
      b; //!< blue
    
    //! ...
    friend Color operator+(const Color& c1,const Color& c2){
      return Color(c1.r+c2.r, c1.g+c2.g, c1.b+c2.b); }
    
    //! ...
    friend Color operator*(float f,const Color& c2){
      return Color(f*c2.r, f*c2.g, f*c2.b); }
    
  public:
    Color();
    Color(uint i);
    Color(float red,float green,float blue);
    Color& operator=(const Color& c);
    bool operator!();
    operator const float*() const;
    void setIndex(unsigned i);
    void setRgb(float red,float green,float blue);
    void setRgbByte(byte red,byte green,byte blue);
    void setHsv(int hue,byte sat,byte val);
    void setTemp(float temp);
    void setTemp2(float temp);
    void setGray(float gray);
    void getRgb(byte& R,byte& G,byte& B) const;
    float getGray() const;
    void whiten(float f);
    void blacken(float f);
    void mix(Color& A,Color& B,float f=.5);
    void mixAdd(Color& A,Color& B,float f=.5);
    void mixSub(Color& A,Color& B,float f=.5);
    void min(Color& A,Color& B);
    void write(std::ostream& os) const;
    void read(std::istream& is);
  };
}
stdPipes(MT::Color);
#endif


//===========================================================================
//
// generic any container
//

struct Any{
  char *tag;
  const char *type;
  void *p;
  uint n;
  char delim;
  virtual ~Any(){};
  //virtual void read(std::ostream &os) = 0;
  virtual void write(std::ostream &os) const = 0;
  virtual void free() = 0;
  virtual Any *new_clone() = 0;
  //virtual void operator=(const Any&) = 0;
  //virtual void set(const void *_p,uint _n) = 0;
};
stdOutPipe(Any);

template<class T> Any* anyNew(const char* tag,const T &x);
template<class T> Any* anyNew(const char* tag,const T *x,uint n,char delim);

#if 0 //don't need the bag anymore, use an any list instead!
    
#define BAG_DEBUG(x)

namespace MT{
  class BagItem;

  /*!\brief a simple any-type container: basically a list of items,
    where an item is a type-identifying tag and a void pointer */
  class Bag{
  public:
    BagItem *first; //!< pointer to first item in the bag

    Bag();
    ~Bag();
    void operator=(const Bag&);
    void clear();
    void write(std::ostream &os) const;
    bool contains(const char *tag);

    static void writeExistingItemTypes(std::ostream& os);
    static int getType(const char *tag,const char *typeidname,uint n);

    /*struct Any{
      template<class T> Any(const char *tag,T *x,uint n){
    };*/
    
    template<class T> void add(const char *tag,T *x,uint n)
#ifdef MT_MSVC
    {
      BagItem *a=new BagTypedItem<T>;
      a->type=getType(tag,typeid(T).name(),n);
      if(a->type==-1){ delete a; return; }
      a->set(x);
      a->next=first;
      first=a;
    }
#else
    ;
#endif

    template<class T> BagItem* get(const char *tag,T *&x,uint n)
#ifdef MT_MSVC
    {
      int i;
      BagItem *a;
      i=getType(tag,typeid(T).name(),n);
      for(a=first;a;a=a->next) if(a->type==i) break;
      if(a) x=(T*)a->value; else x=NULL;
      return a;
    }
#else
    ;
#endif
    
  };
}
#endif

//===========================================================================
//
// gnuplot calls
//

void gnuplot(const char *command,const char* EPSfile=NULL);
void gnuplotClose();


//===========================================================================
//
// implementations
//

#if defined MT_IMPLEMENTATION | defined MT_IMPLEMENT_TEMPLATES
#  include "util_t.cpp"
#endif

#ifdef MT_IMPLEMENTATION
#  include"util.cpp"
#endif



//===========================================================================
//
// USING's
//

using std::cout;
using std::cerr;
using std::endl;
using std::flush;
using std::ostream;
using std::istream;
using std::ofstream;
using std::ifstream;
using MT::rnd;
using MT::Color;
using MT::String;


#endif
